@@ -1,3 +1,18 @@
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*                          Benoit Vaugon, ENSTA                          *)
+(*                                                                        *)
+(*   Copyright 2014 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
+
 (* Padding position. *)
 type padty =
   | Left   (* Text is left justified ('-' option).               *)
@@ -23,6 +38,8 @@ type float_conv =
   | Float_g | Float_pg | Float_sg  (*  %g | %+g | % g  *)
   | Float_G | Float_pG | Float_sG  (*  %G | %+G | % G  *)
   | Float_F                        (*  %F              *)
+  | Float_h | Float_ph | Float_sh  (*  %h | %+h | % h  *)
+  | Float_H | Float_pH | Float_sH  (*  %H | %+H | % H  *)
 
 (***)
 
@@ -65,6 +82,12 @@ type ('a, 'b) precision =
    only accept an optional number as precision option (no extra argument) *)
 type prec_option = int option
 
+(* see the Custom format combinator *)
+type ('a, 'b, 'c) custom_arity =
+  | Custom_zero : ('a, string, 'a) custom_arity
+  | Custom_succ : ('a, 'b, 'c) custom_arity ->
+    ('a, 'x -> 'b, 'x -> 'c) custom_arity
+
 (***)
 
 (*        Relational format types
@@ -84,11 +107,11 @@ position in the format tail (('u, .., 'f) fmt). This means that the
 type of the expected format parameter depends of where the %(...%)
 are in the format string:
 
-  # Printf.printf "%(%)";;
+  # Printf.printf "%(%)"
   - : (unit, out_channel, unit, '_a, '_a, unit)
       CamlinternalFormatBasics.format6 -> unit
   = <fun>
-  # Printf.printf "%(%)%d";;
+  # Printf.printf "%(%)%d"
   - : (int -> unit, out_channel, unit, '_a, '_a, int -> unit)
       CamlinternalFormatBasics.format6 -> int -> unit
   = <fun>
@@ -129,7 +152,7 @@ parameter is as follows:
     (char -> 'a1, 'b1, 'c1, 'd1, 'e1, 'f1,
      char -> 'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel
 
-In the general case, the term structure of fmtty_rel is (almost¹)
+In the general case, the term structure of fmtty_rel is (almost[1])
 isomorphic to the fmtty of the previous implementation: every
 constructor is re-read with a binary, relational type, instead of the
 previous unary typing. fmtty can then be re-defined as the diagonal of
@@ -163,7 +186,7 @@ to transpose between related format types.
       'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel
   -> ('a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmt
 
-NOTE ¹: the typing of Format_subst_ty requires not one format type, but
+NOTE [1]: the typing of Format_subst_ty requires not one format type, but
 two, one to establish the link between the format argument and the
 first six parameters, and the other for the link between the format
 argumant and the last six parameters.
@@ -284,7 +307,8 @@ and ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
       ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
        'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel ->
       (('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'a1, 'b1, 'c1, 'd1, 'e1, 'f1,
-       ('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel
+       ('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'a2, 'b2, 'c2, 'd2, 'e2, 'f2)
+           fmtty_rel
   | Format_subst_ty :                                         (* %(...%) *)
       ('g, 'h, 'i, 'j, 'k, 'l,
        'g1, 'b1, 'c1, 'j1, 'd1, 'a1) fmtty_rel *
@@ -293,7 +317,8 @@ and ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
       ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
        'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel ->
       (('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'g1, 'b1, 'c1, 'j1, 'e1, 'f1,
-       ('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'g2, 'b2, 'c2, 'j2, 'e2, 'f2) fmtty_rel
+       ('g, 'h, 'i, 'j, 'k, 'l) format6 -> 'g2, 'b2, 'c2, 'j2, 'e2, 'f2)
+           fmtty_rel
 
   (* Printf and Format specific constructors. *)
   | Alpha_ty :                                                (* %a  *)
@@ -306,6 +331,11 @@ and ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
        'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel ->
       (('b1 -> 'c1) -> 'a1, 'b1, 'c1, 'd1, 'e1, 'f1,
        ('b2 -> 'c2) -> 'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel
+  | Any_ty :                                    (* Used for custom formats *)
+      ('a1, 'b1, 'c1, 'd1, 'e1, 'f1,
+       'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel ->
+      ('x -> 'a1, 'b1, 'c1, 'd1, 'e1, 'f1,
+       'x -> 'a2, 'b2, 'c2, 'd2, 'e2, 'f2) fmtty_rel
 
   (* Scanf specific constructor. *)
   | Reader_ty :                                               (* %r  *)
@@ -417,6 +447,32 @@ and ('a, 'b, 'c, 'd, 'e, 'f) fmt =
       ('a, 'b, 'c, 'd, 'y, 'x) ignored * ('x, 'b, 'c, 'y, 'e, 'f) fmt ->
         ('a, 'b, 'c, 'd, 'e, 'f) fmt
 
+  (* Custom printing format (PR#6452, GPR#140)
+
+     We include a type Custom of "custom converters", where an
+     arbitrary function can be used to convert one or more
+     arguments. There is no syntax for custom converters, it is only
+     inteded for custom processors that wish to rely on the
+     stdlib-defined format GADTs.
+
+     For instance a pre-processor could choose to interpret strings
+     prefixed with ["!"] as format strings where [%{{ ... }}] is
+     a special form to pass a to_string function, so that one could
+     write:
+
+     {[
+       type t = { x : int; y : int }
+
+       let string_of_t t = Printf.sprintf "{ x = %d; y = %d }" t.x t.y
+
+       Printf.printf !"t = %{{string_of_t}}" { x = 42; y = 42 }
+     ]}
+  *)
+  | Custom :
+      ('a, 'x, 'y) custom_arity * (unit -> 'x) * ('a, 'b, 'c, 'd, 'e, 'f) fmt ->
+      ('y, 'b, 'c, 'd, 'e, 'f) fmt
+
+  (* end of a format specification *)
   | End_of_format :
         ('f, 'b, 'c, 'e, 'e, 'f) fmt
 
@@ -490,6 +546,8 @@ let rec erase_rel : type a b c d e f g h i j k l .
     Alpha_ty (erase_rel rest)
   | Theta_ty rest ->
     Theta_ty (erase_rel rest)
+  | Any_ty rest ->
+    Any_ty (erase_rel rest)
   | Reader_ty rest ->
     Reader_ty (erase_rel rest)
   | Ignored_reader_ty rest ->
@@ -543,6 +601,8 @@ fun fmtty1 fmtty2 -> match fmtty1 with
     Alpha_ty (concat_fmtty rest fmtty2)
   | Theta_ty rest ->
     Theta_ty (concat_fmtty rest fmtty2)
+  | Any_ty rest ->
+    Any_ty (concat_fmtty rest fmtty2)
   | Reader_ty rest ->
     Reader_ty (concat_fmtty rest fmtty2)
   | Ignored_reader_ty rest ->
@@ -588,6 +648,8 @@ fun fmt1 fmt2 -> match fmt1 with
     Alpha (concat_fmt rest fmt2)
   | Theta rest ->
     Theta (concat_fmt rest fmt2)
+  | Custom (arity, f, rest) ->
+    Custom (arity, f, concat_fmt rest fmt2)
   | Reader rest ->
     Reader (concat_fmt rest fmt2)
   | Flush rest ->
