@@ -1,17 +1,17 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                                OCaml                                *)
-(*                                                                     *)
-(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
-(*                                                                     *)
-(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License, with    *)
-(*  the special exception on linking described in file ../LICENSE.     *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: char.ml 11156 2011-07-27 14:17:02Z doligez $ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           *)
+(*                                                                        *)
+(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (* Character operations *)
 
@@ -21,10 +21,7 @@ external unsafe_chr: int -> char = "%identity"
 let chr n =
   if n < 0 || n > 255 then invalid_arg "Char.chr" else unsafe_chr n
 
-external is_printable: char -> bool = "caml_is_printable"
-
 external string_create: int -> string = "caml_create_string"
-external string_unsafe_get : string -> int -> char = "%string_unsafe_get"
 external string_unsafe_set : string -> int -> char -> unit
                            = "%string_unsafe_set"
 
@@ -35,12 +32,11 @@ let escaped = function
   | '\t' -> "\\t"
   | '\r' -> "\\r"
   | '\b' -> "\\b"
-  | c ->
-    if is_printable c then begin
+  | ' ' .. '~' as c ->
       let s = string_create 1 in
       string_unsafe_set s 0 c;
       s
-    end else begin
+  | c ->
       let n = code c in
       let s = string_create 4 in
       string_unsafe_set s 0 '\\';
@@ -48,7 +44,6 @@ let escaped = function
       string_unsafe_set s 2 (unsafe_chr (48 + (n / 10) mod 10));
       string_unsafe_set s 3 (unsafe_chr (48 + n mod 10));
       s
-    end
 
 let lowercase c =
   if (c >= 'A' && c <= 'Z')
@@ -64,6 +59,17 @@ let uppercase c =
   then unsafe_chr(code c - 32)
   else c
 
+let lowercase_ascii c =
+  if (c >= 'A' && c <= 'Z')
+  then unsafe_chr(code c + 32)
+  else c
+
+let uppercase_ascii c =
+  if (c >= 'a' && c <= 'z')
+  then unsafe_chr(code c - 32)
+  else c
+
 type t = char
 
 let compare c1 c2 = code c1 - code c2
+let equal (c1: t) (c2: t) = compare c1 c2 = 0
