@@ -1,21 +1,21 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                                OCaml                                *)
-(*                                                                     *)
-(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
-(*                                                                     *)
-(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License, with    *)
-(*  the special exception on linking described in file ../LICENSE.     *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: digest.mli 12212 2012-03-08 22:27:57Z doligez $ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           *)
+(*                                                                        *)
+(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** MD5 message digest.
 
-   This module provides functions to compute 128-bit ``digests'' of
+   This module provides functions to compute 128-bit 'digests' of
    arbitrary-length strings or files. The digests are of cryptographic
    quality: it is very hard, given a digest, to forge a string having
    that digest. The algorithm used is MD5. This module should not be
@@ -35,13 +35,25 @@ val compare : t -> t -> int
     argument to the functors {!Set.Make} and {!Map.Make}.
     @since 4.00.0 *)
 
+val equal : t -> t -> bool
+(** The equal function for 16-character digest.
+    @since 4.03.0 *)
+
 val string : string -> t
 (** Return the digest of the given string. *)
 
+val bytes : bytes -> t
+(** Return the digest of the given byte sequence.
+    @since 4.02.0 *)
+
 val substring : string -> int -> int -> t
 (** [Digest.substring s ofs len] returns the digest of the substring
-   of [s] starting at character number [ofs] and containing [len]
-   characters. *)
+   of [s] starting at index [ofs] and containing [len] characters. *)
+
+val subbytes : bytes -> int -> int -> t
+(** [Digest.subbytes s ofs len] returns the digest of the subsequence
+    of [s] starting at index [ofs] and containing [len] bytes.
+    @since 4.02.0 *)
 
 external channel : in_channel -> int -> t = "caml_md5_chan"
 (** If [len] is nonnegative, [Digest.channel ic len] reads [len]
