@@ -1,15 +1,17 @@
-(***********************************************************************)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*            Maxence Guesdon, projet Cristal, INRIA Rocquencourt      *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: odoc_info.mli 8932 2008-07-25 13:28:23Z guesdon $ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Maxence Guesdon, projet Cristal, INRIA Rocquencourt        *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Interface to the information collected in source files. *)
 
@@ -21,10 +23,13 @@ type ref_kind = Odoc_types.ref_kind =
   | RK_class_type
   | RK_value
   | RK_type
+  | RK_extension
   | RK_exception
   | RK_attribute
   | RK_method
   | RK_section of text
+  | RK_recfield
+  | RK_const
 
 and text_element = Odoc_types.text_element =
   | Raw of string (** Raw text. *)
@@ -45,14 +50,17 @@ and text_element = Odoc_types.text_element =
              (** Style number, optional label, and text. *)
   | Latex of string (** A string for latex. *)
   | Link of string * text (** A reference string and the link text. *)
-  | Ref of string * ref_kind option
-       (** A reference to an element. Complete name and kind. *)
+  | Ref of string * ref_kind option * text option
+       (** A reference to an element. Complete name and kind.
+        An optional text can be given to display this text instead
+        of the element name.*)
   | Superscript of text (** Superscripts. *)
   | Subscript of text (** Subscripts. *)
   | Module_list of string list
        (** The table of the given modules with their abstract. *)
   | Index_list (** The links to the various indexes (values, types, ...) *)
   | Custom of string * text (** to extend \{foo syntax *)
+  | Target of string * string (** (target, code) : to specify code specific to a target format *)
 
 (** A text is a list of [text_element]. The order matters. *)
 and text = text_element list
@@ -76,13 +84,16 @@ type param = (string * text)
 (** Raised exception name and description. *)
 type raised_exception = (string * text)
 
-(** Information in a special comment *)
+(** Information in a special comment
+@before 3.12.0 \@before information was not present.
+*)
 type info = Odoc_types.info = {
     i_desc : text option; (** The description text. *)
     i_authors : string list; (** The list of authors in \@author tags. *)
     i_version : string option; (** The string in the \@version tag. *)
     i_sees : see list; (** The list of \@see tags. *)
     i_since : string option; (** The string in the \@since tag. *)
+    i_before : (string * text) list ; (** the version number and text in \@before tag *)
     i_deprecated : text option; (** The of the \@deprecated tag. *)
     i_params : param list; (** The list of parameter descriptions. *)
     i_raised_exceptions : raised_exception list; (** The list of raised exceptions. *)
@@ -92,8 +103,8 @@ type info = Odoc_types.info = {
 
 (** Location of elements in implementation and interface files. *)
 type location = Odoc_types.location = {
-    loc_impl : (string * int) option ; (** implementation file name and position *)
-    loc_inter : (string * int) option ; (** interface file name and position *)
+    loc_impl : Location.t option ; (** implementation location *)
+    loc_inter : Location.t option ; (** interface location *)
   }
 
 (** A dummy location. *)
@@ -162,6 +173,47 @@ module Parameter :
     val type_by_name : parameter -> string -> Types.type_expr
   end
 
+(** Representation and manipulation of extensions. *)
+module Extension :
+  sig
+    type private_flag = Odoc_extension.private_flag =
+      Private | Public
+
+    (** Used when the extension is a rebind of another extension,
+       when we have [extension Xt = Target_xt].*)
+    type extension_alias = Odoc_extension.extension_alias =
+        {
+          xa_name : Name.t ; (** The complete name of the target extension. *)
+          mutable xa_xt : t_extension_constructor option ; (** The target extension, if we found it.*)
+        }
+
+    and t_extension_constructor = Odoc_extension.t_extension_constructor =
+        {
+          xt_name : Name.t ;
+          xt_args: Odoc_type.constructor_args;
+          xt_ret: Types.type_expr option ; (** the optional return type of the extension *)
+          xt_type_extension: t_type_extension ; (** the type extension containing this constructor *)
+          xt_alias: extension_alias option ; (** [None] when the extension is not a rebind. *)
+          mutable xt_loc: Odoc_types.location ;
+          mutable xt_text: Odoc_types.info option ; (** optional user description *)
+        }
+
+    and t_type_extension = Odoc_extension.t_type_extension =
+        {
+          mutable te_info : info option ; (** Information found in the optional associated comment. *)
+          te_type_name : Name.t ; (** The type of the extension *)
+          te_type_parameters : Types.type_expr list;
+          te_private : private_flag ;
+          mutable te_constructors: t_extension_constructor list;
+          mutable te_loc : location ;
+          mutable te_code : string option ;
+        }
+
+    (** Access to the extensions in a group. *)
+    val extension_constructors : t_type_extension -> t_extension_constructor list
+
+  end
+
 (** Representation and manipulation of exceptions. *)
 module Exception :
   sig
@@ -177,7 +229,8 @@ module Exception :
         {
           ex_name : Name.t ;
           mutable ex_info : info option ; (** Information found in the optional associated comment. *)
-          ex_args : Types.type_expr list ; (** The types of the parameters. *)
+          ex_args : Odoc_type.constructor_args;
+          ex_ret : Types.type_expr option ; (** The the optional return type of the exception. *)
           ex_alias : exception_alias option ; (** [None] when the exception is not a rebind. *)
           mutable ex_loc : location ;
           mutable ex_code : string option ;
@@ -190,21 +243,26 @@ module Type :
     type private_flag = Odoc_type.private_flag =
       Private | Public
 
-    (** Description of a variant type constructor. *)
-    type variant_constructor = Odoc_type.variant_constructor =
-        {
-          vc_name : string ; (** Name of the constructor. *)
-          vc_args : Types.type_expr list ; (** Arguments of the constructor. *)
-          mutable vc_text : text option ; (** Optional description in the associated comment. *)
-        }
-
     (** Description of a record type field. *)
     type record_field = Odoc_type.record_field =
         {
           rf_name : string ; (** Name of the field. *)
           rf_mutable : bool ; (** [true] if mutable. *)
           rf_type : Types.type_expr ; (** Type of the field. *)
-          mutable rf_text : text option ; (** Optional description in the associated comment.*)
+          mutable rf_text : info option ; (** Optional description in the associated comment.*)
+        }
+
+    (** Description of a variant type constructor. *)
+    type constructor_args = Odoc_type.constructor_args =
+      | Cstr_record of record_field list
+      | Cstr_tuple of Types.type_expr list
+
+    type variant_constructor = Odoc_type.variant_constructor =
+        {
+          vc_name : string ; (** Name of the constructor. *)
+          vc_args : constructor_args;
+          vc_ret : Types.type_expr option ;
+          mutable vc_text : info option ; (** Optional description in the associated comment. *)
         }
 
     (** The various kinds of a type. *)
@@ -214,6 +272,17 @@ module Type :
                    (** constructors *)
       | Type_record of record_field list
                    (** fields *)
+      | Type_open (** Type is open *)
+
+    type object_field = Odoc_type.object_field = {
+      of_name : string ;
+      of_type : Types.type_expr ;
+      mutable of_text : Odoc_types.info option ; (** optional user description *)
+    }
+
+    type type_manifest = Odoc_type.type_manifest =
+      | Other of Types.type_expr (** Type manifest directly taken from Typedtre. *)
+      | Object_type of object_field list
 
     (** Representation of a type. *)
     type t_type = Odoc_type.t_type =
@@ -224,7 +293,7 @@ module Type :
                     (** type parameters: (type, covariant, contravariant) *)
           ty_kind : type_kind; (** Type kind. *)
           ty_private : private_flag; (** Private or public type. *)
-          ty_manifest : Types.type_expr option; (** Type manifest. *)
+          ty_manifest : type_manifest option ;
           mutable ty_loc : location ;
           mutable ty_code : string option;
         }
@@ -338,7 +407,7 @@ module Class :
         {
           cta_name : Name.t ; (** Complete name of the target class type. *)
           mutable cta_class : cct option ;  (** The target t_class or t_class_type, if we found it.*)
-          cta_type_parameters : Types.type_expr list ; (** The type parameters. A VOIR : mettre des string ? *)
+          cta_type_parameters : Types.type_expr list ; (** The type parameters. FIXME : use strings? *)
         }
 
     and class_type_kind = Odoc_class.class_type_kind =
@@ -402,6 +471,7 @@ module Module :
       | Element_class of Class.t_class
       | Element_class_type of Class.t_class_type
       | Element_value of Value.t_value
+      | Element_type_extension of Extension.t_type_extension
       | Element_exception of Exception.t_exception
       | Element_type of Type.t_type
       | Element_module_comment of text
@@ -426,7 +496,7 @@ module Module :
 
     and module_parameter = Odoc_module.module_parameter = {
         mp_name : string ; (** the name *)
-        mp_type : Types.module_type ; (** the type *)
+        mp_type : Types.module_type option ; (** the type *)
         mp_type_code : string ; (** the original code *)
         mp_kind : module_type_kind ; (** the way the parameter was built *)
       }
@@ -444,6 +514,8 @@ module Module :
                         Should appear in interface files only. *)
       | Module_constraint of module_kind * module_type_kind
                      (** A module constraint by a module type. *)
+      | Module_typeof of string (** by now only the code of the module expression *)
+      | Module_unpack of string * module_type_alias (** code of the expression and module type alias *)
 
     (** Representation of a module. *)
     and t_module = Odoc_module.t_module =
@@ -476,6 +548,8 @@ module Module :
             (** Complete alias name and corresponding module type if we found it. *)
       | Module_type_with of module_type_kind * string
             (** The module type kind and the code of the with constraint. *)
+      | Module_type_typeof of string
+            (** by now only the code of the module expression *)
 
     (** Representation of a module type. *)
     and t_module_type = Odoc_module.t_module_type =
@@ -506,6 +580,9 @@ module Module :
     (** Access to the included modules of a module. *)
     val module_included_modules : ?trans:bool-> t_module -> included_module list
 
+    (** Access to the type extensions of a module. *)
+    val module_type_extensions : ?trans:bool-> t_module -> Extension.t_type_extension list
+
     (** Access to the exceptions of a module. *)
     val module_exceptions : ?trans:bool-> t_module -> Exception.t_exception list
 
@@ -618,6 +695,10 @@ val string_of_type_list : ?par: bool -> string -> Types.type_expr list -> string
    for the given type. *)
 val string_of_type_param_list : Type.t_type -> string
 
+(** This function returns a string to represent the list of type parameters
+   for the given type extension. *)
+val string_of_type_extension_param_list : Extension.t_type_extension -> string
+
 (** This function returns a string to represent the given list of
    type parameters of a class or class type,
    with a given separator. *)
@@ -647,6 +728,11 @@ val string_of_info : info -> string
 (** @return a string to describe the given type. *)
 val string_of_type : Type.t_type -> string
 
+val string_of_record : Type.record_field list -> string
+
+(** @return a string to describe the given type extension. *)
+val string_of_type_extension : Extension.t_type_extension -> string
+
 (** @return a string to describe the given exception. *)
 val string_of_exception : Exception.t_exception -> string
 
@@ -697,11 +783,11 @@ val create_index_lists : 'a list -> ('a -> string) -> 'a list list
 val remove_option : Types.type_expr -> Types.type_expr
 
 (** Return [true] if the given label is optional.*)
-val is_optional : string -> bool
+val is_optional : Asttypes.arg_label -> bool
 
 (** Return the label name for the given label,
    i.e. removes the beginning '?' if present.*)
-val label_name : string -> string
+val label_name : Asttypes.arg_label -> string
 
 (** Return the given name where the module name or
    part of it was removed, according to the list of modules
@@ -777,10 +863,13 @@ module Search :
         | Res_class_type of Class.t_class_type
         | Res_value of Value.t_value
         | Res_type of Type.t_type
+        | Res_extension of Extension.t_extension_constructor
         | Res_exception of Exception.t_exception
         | Res_attribute of Value.t_attribute
         | Res_method of Value.t_method
         | Res_section of string  * text
+        | Res_recfield of Type.t_type * Type.record_field
+        | Res_const of Type.t_type * Type.variant_constructor
 
       (** The type representing a research result.*)
       type search_result = result_element list
@@ -791,6 +880,9 @@ module Search :
       (** A function to search all the values in a list of modules. *)
       val values : Module.t_module list -> Value.t_value list
 
+      (** A function to search all the extensions in a list of modules. *)
+      val extensions : Module.t_module list -> Extension.t_extension_constructor list
+
       (** A function to search all the exceptions in a list of modules. *)
       val exceptions : Module.t_module list -> Exception.t_exception list
 
@@ -825,12 +917,32 @@ module Scan :
       (** Scan of 'leaf elements'. *)
 
         method scan_value : Value.t_value -> unit
+
+        method scan_type_pre : Type.t_type -> bool
+        method scan_type_const : Type.t_type -> Type.variant_constructor -> unit
+        method scan_type_recfield : Type.t_type -> Type.record_field -> unit
         method scan_type : Type.t_type -> unit
+        method scan_extension_constructor : Extension.t_extension_constructor -> unit
         method scan_exception : Exception.t_exception -> unit
         method scan_attribute : Value.t_attribute -> unit
         method scan_method : Value.t_method -> unit
         method scan_included_module : Module.included_module -> unit
 
+      (** Scan of a type extension *)
+
+        (** Overide this method to perform controls on the extension's type,
+            private and info. This method is called before scanning the
+            extension's constructors.
+            @return true if the extension's constructors must be scanned.*)
+        method scan_type_extension_pre : Extension.t_type_extension -> bool
+
+       (** This method scans the constructors of the given type extension. *)
+        method scan_type_extension_constructors : Extension.t_type_extension -> unit
+
+        (** Scan of a type extension. Should not be overridden. It calls [scan_type_extension_pre]
+            and if [scan_type_extension_pre] returns [true], then it calls scan_type_extension_constructors.*)
+        method scan_type_extension : Extension.t_type_extension -> unit
+
       (** Scan of a class. *)
 
         (** Scan of a comment inside a class. *)
@@ -844,7 +956,7 @@ module Scan :
        (** This method scan the elements of the given class. *)
         method scan_class_elements : Class.t_class -> unit
 
-       (** Scan of a class. Should not be overriden. It calls [scan_class_pre]
+       (** Scan of a class. Should not be overridden. It calls [scan_class_pre]
           and if [scan_class_pre] returns [true], then it calls scan_class_elements.*)
         method scan_class : Class.t_class -> unit
 
@@ -861,7 +973,7 @@ module Scan :
         (** This method scan the elements of the given class type. *)
         method scan_class_type_elements : Class.t_class_type -> unit
 
-        (** Scan of a class type. Should not be overriden. It calls [scan_class_type_pre]
+        (** Scan of a class type. Should not be overridden. It calls [scan_class_type_pre]
            and if [scan_class_type_pre] returns [true], then it calls scan_class_type_elements.*)
         method scan_class_type : Class.t_class_type -> unit
 
@@ -878,7 +990,7 @@ module Scan :
         (** This method scan the elements of the given module. *)
         method scan_module_elements : Module.t_module -> unit
 
-       (** Scan of a module. Should not be overriden. It calls [scan_module_pre]
+       (** Scan of a module. Should not be overridden. It calls [scan_module_pre]
           and if [scan_module_pre] returns [true], then it calls scan_module_elements.*)
         method scan_module : Module.t_module -> unit
 
@@ -895,7 +1007,7 @@ module Scan :
         (** This method scan the elements of the given module type. *)
         method scan_module_type_elements : Module.t_module_type -> unit
 
-        (** Scan of a module type. Should not be overriden. It calls [scan_module_type_pre]
+        (** Scan of a module type. Should not be overridden. It calls [scan_module_type_pre]
            and if [scan_module_type_pre] returns [true], then it calls scan_module_type_elements.*)
         method scan_module_type : Module.t_module_type -> unit
 
@@ -921,149 +1033,40 @@ module Dep :
     val deps_of_types : ?kernel: bool -> Type.t_type list -> (Type.t_type * (Name.t list)) list
   end
 
-(** {2 Command line arguments} *)
+(** {2 Some global variables} *)
 
-(**  You can use this module to create custom generators.*)
-module Args :
-    sig
-      (** The kind of source file in arguments. *)
-      type source_file =
-          Impl_file of string
-        | Intf_file of string
-        | Text_file of string
+module Global :
+  sig
+    val errors : int ref
+    val warn_error : bool ref
 
-      (** The class type of documentation generators. *)
-      class type doc_generator =
-        object method generate : Module.t_module list -> unit end
+    (** The file used by the generators outputting only one file. *)
+    val out_file : string ref
 
-      (** The file used by the generators outputting only one file. *)
-      val out_file : string ref
+    (** Verbose mode or not. *)
+    val verbose : bool ref
 
-      (** Verbose mode or not. *)
-      val verbose : bool ref
+    (** The directory where files have to be generated. *)
+    val target_dir : string ref
 
-      (** The optional title to use in the generated documentation. *)
-      val title : string option ref
+    (** The optional title to use in the generated documentation. *)
+    val title : string option ref
 
-      (** To inverse [.ml] and [.mli] files while merging comments. *)
-      val inverse_merge_ml_mli : bool ref
+    (** The optional file whose content can be used as intro text. *)
+    val intro_file : string option ref
 
-      (** To filter module elements according to module type constraints. *)
-      val filter_with_module_constraints : bool ref
+    (** The flag which indicates if we must generate a table of contents. *)
+    val with_toc : bool ref
 
-      (** To keep the code while merging, when we have both .ml and .mli files for a module. *)
-      val keep_code : bool ref
+    (** The flag which indicates if we must generate an index. *)
+    val with_index : bool ref
 
-      (** The optional file whose content can be used as intro text. *)
-      val intro_file : string option ref
+    (** The flag which indicates if we must generate a header.*)
+    val with_header : bool ref
 
-      (** Flag to indicate whether we must display the complete list of parameters
-         for functions and methods. *)
-      val with_parameter_list : bool ref
-
-      (** The list of module names to hide. *)
-      val hidden_modules : string list ref
-
-      (** The directory where files have to be generated. *)
-      val target_dir : string ref
-
-      (** An optional file to use where a CSS style is defined (for HTML). *)
-      val css_style : string option ref
-
-      (** Generate only index files. (for HTML). *)
-      val index_only : bool ref
-
-      (** To colorize code in HTML generated documentation pages, not code pages. *)
-      val colorize_code : bool ref
-
-      (** To display functors in short form rather than with "functor ... -> ",
-         in HTML generated documentation. *)
-      val html_short_functors : bool ref
-
-      (** The flag which indicates if we must generate a header (for LaTeX). *)
-      val with_header : bool ref
-
-      (** The flag which indicates if we must generate a trailer (for LaTeX). *)
-      val with_trailer : bool ref
-
-      (** The flag to indicate if we must generate one file per module (for LaTeX). *)
-      val separate_files : bool ref
-
-      (** The list of pairs (title level, sectionning style). *)
-      val latex_titles : (int * string) list ref
-
-      (** The prefix to use for value labels in LaTeX. *)
-      val latex_value_prefix : string ref
-
-      (** The prefix to use for type labels in LaTeX. *)
-      val latex_type_prefix : string ref
-
-      (** The prefix to use for exception labels in LaTeX. *)
-      val latex_exception_prefix : string ref
-
-      (** The prefix to use for module labels in LaTeX. *)
-      val latex_module_prefix : string ref
-
-      (** The prefix to use for module type labels in LaTeX. *)
-      val latex_module_type_prefix : string ref
-
-      (** The prefix to use for class labels in LaTeX. *)
-      val latex_class_prefix : string ref
-
-      (** The prefix to use for class type labels in LaTeX. *)
-      val latex_class_type_prefix : string ref
-
-      (** The prefix to use for attribute labels in LaTeX. *)
-      val latex_attribute_prefix : string ref
-
-      (** The prefix to use for method labels in LaTeX. *)
-      val latex_method_prefix : string ref
-
-      (** The flag which indicates if we must generate a table of contents (for LaTeX). *)
-      val with_toc : bool ref
-
-      (** The flag which indicates if we must generate an index (for TeXinfo). *)
-      val with_index : bool ref
-
-      (** The flag which indicates if we must escape accentuated characters (for TeXinfo).*)
-      val esc_8bits : bool ref
-
-      (** The Info directory section *)
-      val info_section : string ref
-
-      (** The Info directory entries to insert *)
-      val info_entry : string list ref
-
-      (** Include all modules or only the ones on the command line, for the dot ouput. *)
-      val dot_include_all : bool ref
-
-      (** Generate dependency graph for types. *)
-      val dot_types : bool ref
-
-      (** Perform transitive reduction before dot output. *)
-      val dot_reduce : bool ref
-
-      (** The colors used in the dot output. *)
-      val dot_colors : string list ref
-
-      (** The suffix for man pages. *)
-      val man_suffix : string ref
-
-      (** The section for man pages. *)
-      val man_section : string ref
-
-      (** The flag to generate all man pages or only for modules and classes.*)
-      val man_mini : bool ref
-
-      (** The files to be analysed. *)
-      val files : source_file list ref
-
-      (** To set the documentation generator. *)
-      val set_doc_generator : doc_generator option -> unit
-
-      (** Add an option specification. *)
-      val add_option : string * Arg.spec * string -> unit
-    end
+    (** The flag which indicates if we must generate a trailer.*)
+    val with_trailer : bool ref
+end
 
 (** Analysis of the given source files.
    @param init is the list of modules already known from a previous analysis.
@@ -1075,7 +1078,7 @@ val analyse_files :
           ?sort_modules:bool ->
             ?no_stop:bool ->
               ?init: Odoc_module.t_module list ->
-                Args.source_file list ->
+                Odoc_global.source_file list ->
                   Module.t_module list
 
 (** Dump of a list of modules into a file.
